unit main;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, xpman, ExtCtrls, StdCtrls, fx_core, ComCtrls, jpeg, 
  Buttons, MMSystem;

type
  TfrmMain = class(TForm)
    Panel1: TPanel;
    Panel2: TPanel;
    listCat: TListBox;
    Splitter1: TSplitter;
    listFX: TListBox;
    panelSetting: TPanel;
    Splitter2: TSplitter;
    Splitter3: TSplitter;
    Image1: TImage;
    Panel3: TPanel;
    btnBench: TSpeedButton;
    labelBench: TLabel;
    Image2: TImage;
    panelTransition: TPanel;
    btnPlay: TSpeedButton;
    Label1: TLabel;
    TransitionTimer: TTimer;
    procedure FormCreate(Sender: TObject);
    procedure listCatClick(Sender: TObject);
    procedure listFXClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure btnBenchClick(Sender: TObject);
    procedure btnPlayClick(Sender: TObject);
    procedure TransitionTimerTimer(Sender: TObject);
    procedure FormPaint(Sender: TObject);
  private
    { Private declarations }
    FDown: TPoint;
    procedure DoFloatTrackChange(Sender: TObject);
    procedure DoIntTrackChange(Sender: TObject);
    procedure DoShapeMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure DoShapeMouseMove(Sender: TObject; Shift: TShiftState;
      X, Y: Integer);
    procedure DoShapeMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure DoButtonClick(Sender: TObject);
    procedure UpdatePreview;
    procedure UpdatePoints;
  public
    { Public declarations }
    Filter: TfxFilter;
    Rec: TfxFilterRec;
    Source: TBitmap;
    Target: TBitmap;
    Dest: TBitmap;
    ProgressTrack: TTrackBar;
    Dx: integer;
  end;

var
  frmMain: TfrmMain;

implementation

{$R *.dfm}

procedure TfrmMain.FormCreate(Sender: TObject);
begin
  Source := TBitmap.Create;
  Source.Assign(Image1.Picture.Graphic);
  Target := TBitmap.Create;
  Target.Assign(Image2.Picture.Graphic);

  FillCategory(listCat.Items);
end;

procedure TfrmMain.FormDestroy(Sender: TObject);
begin
  if Filter <> nil then
    Filter.Free;
  Target.Free;
  Source.Free;
end;

procedure TfrmMain.listCatClick(Sender: TObject);
begin
  if TransitionTimer.Enabled then
    btnPlayClick(Self);
  if listCat.ItemIndex < 0 then Exit;
  FillFiltersInCategory(listCat.Items[listCat.ItemIndex], listFX.Items);
end;

procedure TfrmMain.listFXClick(Sender: TObject);
var
  i: integer;
  Layout: TPanel;
  Text: TLabel;
  Track: TTrackBar;
  Shape: TPanel;
  Button: TButton;
begin
  if TransitionTimer.Enabled then
    btnPlayClick(Self);
  if listFX.ItemIndex < 0 then Exit;
  panelTransition.Visible := listCat.Items[listCat.ItemIndex] = 'Transition';
  // Delete setting controls
  for i := panelSetting.ControlCount - 1 downto 0 do
    panelSetting.Controls[i].Free;
  for i := ControlCount - 1 downto 0 do
  begin
    if Controls[i].Tag = 0 then Continue;
    Controls[i].Free;
  end;
  // Empty Dest
  Dest := nil;
  //
  ProgressTrack := nil;
  // Create Filter
  if Filter <> nil then
    Filter.Free;
  Filter := FilterByName(listFX.Items[listFX.ItemIndex]);
  // Create settings
  Rec := FilterClassByName(listFX.Items[listFX.ItemIndex]).FilterAttr;
  with TLabel.Create(Self) do
  begin
    Parent := panelSetting;
    AutoSize := false;
    Height := 51;
    Align := alTop;
    Caption := Rec.Desc;
    WordWrap := true;
  end;
  for i := 0 to High(Rec.Values) do
  with Rec.Values[i] do
  begin
    case ValueType of
      feValueFloat:
        begin
          Layout := TPanel.Create(Self);
          Layout.Parent := panelSetting;
          Layout.BevelOuter := bvNone;
          Layout.Height := 25;
          Layout.Top := 1000;
          Layout.Align := alTop;
          Text := TLabel.Create(Self);
          Text.Parent := Layout;
          Text.AutoSize := false;
          Text.Width := 110;
          Text.Align := alLeft;
          Text.Caption := Name + ' (' + FloatToStr(Min) + '..' + FloatToStr(Max) + ')';
          Track := TTrackBar.Create(Self);
          Track.Parent := Layout;
          Track.Align := alClient;
          Track.ThumbLength := 18;
          Track.TickStyle := tsNone;
          Track.Min := Min * 100;
          Track.Max := Max * 100;
          Track.Position := Default * 100;
          Track.OnChange := DoFloatTrackChange;
          Track.Tag := i;
          if Name = 'Progress' then
            ProgressTrack := Track;
        end;
      feValueInt:
        begin
          Layout := TPanel.Create(Self);
          Layout.Parent := panelSetting;
          Layout.BevelOuter := bvNone;
          Layout.Height := 25;
          Layout.Top := 1000;
          Layout.Align := alTop;
          Text := TLabel.Create(Self);
          Text.Parent := Layout;
          Text.AutoSize := false;
          Text.Width := 110;
          Text.Align := alLeft;
          Text.Caption := Name + ' (' + FloatToStr(Min) + '..' + FloatToStr(Max) + ')';
          Track := TTrackBar.Create(Self);
          Track.Parent := Layout;
          Track.Align := alClient;
          Track.ThumbLength := 18;
          Track.TickStyle := tsNone;
          Track.Min := Min;
          Track.Max := Max;
          Track.Position := Default;
          Track.OnChange := DoIntTrackChange;
          Track.Tag := i;
        end;
      feValuePoint:
        begin
          Shape := TPanel.Create(Self);
          Shape.Parent := Self;
          Shape.Width := 12;
          Shape.Height := 12;
          Shape.Color := clRed;
          Shape.OnMouseDown := DoShapeMouseDown;
          Shape.OnMouseMove := DoShapeMouseMove;
          Shape.OnMouseUp := DoShapeMouseUp;
          Shape.Left := Splitter3.Left + Splitter3.Width + round(VarToPoint(Default).x) - (Shape.Width div 2);
          Shape.Top := 0 + round(VarToPoint(Default).y) - (Shape.Width div 2);
          Shape.Tag := i + 1;
        end;
      feValueColor:
        begin
          Layout := TPanel.Create(Self);
          Layout.Parent := panelSetting;
          Layout.BevelOuter := bvNone;
          Layout.Height := 25;
          Layout.Top := 1000;
          Layout.Align := alTop;
          Text := TLabel.Create(Self);
          Text.Parent := Layout;
          Text.AutoSize := false;
          Text.Width := 90;
          Text.Align := alLeft;
          Text.Caption := Name;
          Button := TButton.Create(Self);
          Button.Parent := Layout;
          Button.Align := alClient;
          Button.OnClick := DoButtonClick;
        end;
    end;
  end;
  UpdatePreview;
end;

procedure TfrmMain.UpdatePoints;
var
  i: integer;
begin
exit;
  for i := ControlCount - 1 downto 0 do
  begin
    if Controls[i].Tag = 0 then Continue;
    with VarToPoint(Filter.Values[Rec.Values[Controls[i].Tag - 1].Name]) do
    begin
      Controls[i].Left := Splitter3.Left + Splitter3.Width + round(x * Source.Width) - (Controls[i].Width div 2);
      Controls[i].Top := 0 + round(y * Source.Height) - (Controls[i].Width div 2);
    end;
  end;
end;

procedure TfrmMain.DoFloatTrackChange(Sender: TObject);
begin
  Filter.Values[Rec.Values[TTrackBar(Sender).Tag].Name] := TTrackBar(Sender).Position / 100;
  UpdatePreview;
  UpdatePoints;
end;

procedure TfrmMain.DoIntTrackChange(Sender: TObject);
begin
  Filter.Values[Rec.Values[TTrackBar(Sender).Tag].Name] := TTrackBar(Sender).Position;
  UpdatePreview;
  UpdatePoints;
end;

procedure TfrmMain.DoButtonClick(Sender: TObject);
var
  C: TColorDialog;
begin
  C := TColorDialog.Create(Self);
  C.Color := $FFFFFF and (Filter.Values[Rec.Values[TTrackBar(Sender).Tag].Name]);
  if C.Execute then
  begin
    Filter.Values[Rec.Values[TTrackBar(Sender).Tag].Name] := C.Color or $FF000000;;
    UpdatePreview;
  end;
end;

procedure TfrmMain.DoShapeMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  SetCapture(TWinControl(Sender).Handle);
  FDown := Point(X, Y);
end;

procedure TfrmMain.DoShapeMouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
var
  P: TPoint;
begin
  if ssLeft in Shift then
  begin
    TPanel(Sender).Left := TPanel(Sender).Left  + (X - FDown.X);
    TPanel(Sender).Top := TPanel(Sender).Top  + (Y - FDown.Y);

    P.X := TPanel(Sender).Left + (TPanel(Sender).Width div 2) - Splitter3.Left - Splitter3.Width;
    P.Y := TPanel(Sender).Top + (TPanel(Sender).Width div 2) - 0;
    Filter.Values[Rec.Values[TPanel(Sender).Tag - 1].Name] := VarFromPointXY(P.X, P.Y);
    UpdatePreview;
  end;
end;

procedure TfrmMain.DoShapeMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  ReleaseCapture;
end;

procedure TfrmMain.UpdatePreview;
begin
  if Filter <> nil then
  begin
    // set input
    Filter.ValuesAsBitmap['Input'] := Source;
    // set Target only for transition
    Filter.ValuesAsBitmap['Target'] := Target;
    // apply, get and show result
    Dest := TBitmap(Filter.ValuesAsBitmap['output']);
    // paint
    FormPaint(Self);
  end;
end;

procedure TfrmMain.btnBenchClick(Sender: TObject);
const
  Steps = 200;
var
  i: integer;
  StartTime, NewTime, Time: single;
begin
  btnBench.Enabled := false;
  StartTime := {$IFDEF WIN32}timeGetTime;{$else}GetTickCount / 1000; {$endif}
  for i := 1 to Steps do
  begin
    if Filter <> nil then
    begin
      // set input
      Filter.ValuesAsBitmap['input'] := Source;
      // apply (usually apply method called when you order Output }
      Filter.Apply;
    end;
  end;
  NewTime := {$IFDEF WIN32}timeGetTime;{$else}GetTickCount / 1000; {$endif}
  labelBench.Caption := 'Average: ' + FloatToStr((NewTime - StartTime) / Steps) + 'ms';
  btnBench.Enabled := true;
end;

procedure TfrmMain.btnPlayClick(Sender: TObject);
begin
  { Play Transition }
  if btnPlay.Caption = 'Play' then
  begin
    Dx := 5;
    TransitionTimer.Tag := ProgressTrack.Position div 100;
    TransitionTimer.Enabled := true;
    btnPlay.Caption := 'Stop';
  end
  else
  begin
    TransitionTimer.Enabled := false;
    btnPlay.Caption := 'Play';
  end;
end;

procedure TfrmMain.TransitionTimerTimer(Sender: TObject);
begin
  ProgressTrack.Position := TransitionTimer.Tag * 100;
  TransitionTimer.Tag := TransitionTimer.Tag + Dx;
  if (TransitionTimer.Tag > 100) or (TransitionTimer.Tag < 0) then
    Dx := -Dx;
end;

procedure TfrmMain.FormPaint(Sender: TObject);
var
  Idx: cardinal;
begin
  if Dest <> nil then
  begin
    Idx := SaveDC(Canvas.Handle);
    ExcludeClipRect(Canvas.Handle, Splitter3.Left + Splitter3.Width, 0, Splitter3.Left + Splitter3.Width + Dest.Width, Dest.Height);
    Canvas.Brush.Color := clGray;
    Canvas.FillRect(Rect(Splitter3.Left + Splitter3.Width, 0, ClientWidth, ClientHeight));
    RestoreDC(Canvas.Handle, Idx);

    Canvas.Draw(Splitter3.Left + Splitter3.Width, 0, Dest)
  end
  else
    Canvas.Draw(Splitter3.Left + Splitter3.Width, 0, Source)
end;

end.
